/*
 * Ratiometric Frequency Counter, LED version
 *
 * ports.h - hardware definition
 *
 * (c) Axel (XL) Schwenke, axel.schwenke@gmx.net
 *
 * $Id: ports.h 123 2011-02-23 23:25:32Z schwenke $
 */

#ifndef PORTS_H
#define PORTS_H

/*
             ATMEGA8 Pinout
            -----------------
      ISP |Res       ADC5/PC5| Analog
      DS0 |PD0            PC4| DB4
      DS1 |PD1            PC3| DB3
      DS2 |PD2            PC2| DB2
      OE  |PD3            PC1| DB1
      FX  |PD4/T0         PC0| DB0
      +5V |Vcc            GND| GND
      GND |GND           ARef| +5V (filtered)
     XTAL |XTAL1         AVcc| +5V (filtered)
     XTAL |XTAL2          SCK| ISP
      DB5 |PD5           MISO| ISP
      DB6 |PD6           MOSI| ISP
      DB7 |PD7            PB2| Clear
     Gate |PB0/ICP1       PB1| Start
           ------------------

DB0 - DB7 .... Data Bus for 74HC590 and LED
DS0 - DS2 .... Digit Select 0-2 (to digit decoder/driver)
OE ........... OE/LAT for 74HC590
FX ........... 9th bit f_x Counter (1/2 74HC74)
Gate ......... Gate Output (1/2 74HC74)
Start ........ Gate Control Line
Clear ........ Clear 74HC590 and Gate
ISP .......... to ISP connector
Analog ....... calibration input

*/

#define GATE_PIN         0      /* Port B */
#define START_PIN        1      /* Port B */
#define CLEAR_PIN        2      /* Port B */
#define LAT_OE_PIN       3      /* Port D */
#define FX_PIN           4      /* Port D */
#define ANALOG_PIN       5      /* Port C */

#define clear_high()     PORTB |= _BV(CLEAR_PIN)
#define clear_low()      PORTB &= ~_BV(CLEAR_PIN)
#define start_high()     PORTB |= _BV(START_PIN)
#define start_low()      PORTB &= ~_BV(START_PIN)
#define lat_oe_high()    PORTD |=  _BV(LAT_OE_PIN)
#define lat_oe_low()     PORTD &= ~_BV(LAT_OE_PIN)

#define gate_is_open()   ((PINB & _BV(GATE_PIN))==0)
#define gate_is_closed() (PINB & _BV(GATE_PIN))
#define fx9_read()       ((PIND >> FX_PIN) & 1)

#define data_output()    DDRC |=  0x1F; DDRD |=  0xE0
#define data_input()     DDRC &= ~0x1F; DDRD &= ~0xE0
#define data_write(x)    PORTC = (PORTC & ~0x1F) | ((x) & 0x1F); \
                         PORTD = (PORTD & ~0xE0) | ((x) & 0xE0)
#define data_read()      ((PINC & 0x1F) | (PIND & 0xE0))


/* PD0,1,2 select which digit is turned on
 * 000=MSD ... 101=LSD, 110=unit LEDs, 111= off */
#define digits_off()	 PORTD |= 0x07
#define digit_on(x)      PORTD = (PORTD & ~0x07) | ((x) & 0x07)
#define digit_read()     (PORTD & 0x07)

/* segment codes */
#define SegA             0x01
#define SegB             0x02
#define SegC             0x04
#define SegD             0x08
#define SegE             0x10
#define SegF             0x20
#define SegG             0x40
#define SegDP            0x80

/* which segment bits are to be inverted */
#define SegInversion     0xFF

/* unit codes */
#define U_Hz             0x01
#define U_kHz            0x02
#define U_MHz            0x04
#define U_OVR            0x80

#endif
